@@ -23,6 +23,7 @@ docs = [
     "rc.md",
     "overview.md",
     "flags.md",
+    "docker.md",
 
     # Keep these alphabetical by full name
     "fichier.md",
@@ -35,11 +36,14 @@ docs = [
     "chunker.md",
     "sharefile.md",
     "crypt.md",
+    "compress.md",
     "dropbox.md",
+    "filefabric.md",
     "ftp.md",
     "googlecloudstorage.md",
     "drive.md",
     "googlephotos.md",
+    "hdfs.md",
     "http.md",
     "hubic.md",
     "jottacloud.md",
@@ -59,9 +63,11 @@ docs = [
     "sftp.md",
     "sugarsync.md",
     "tardigrade.md",
+    "uptobox.md",
     "union.md",
     "webdav.md",
     "yandex.md",
+    "zoho.md",
 
     "local.md",
     "changelog.md",
