@@ -49,10 +49,11 @@ type Marcher interface {
 }
 
 // init sets up a march over opt.Fsrc, and opt.Fdst calling back callback for each match
-func (m *March) init() {
-	m.srcListDir = m.makeListDir(m.Fsrc, m.SrcIncludeAll)
+func (m *March) init(ctx context.Context) {
+	ci := fs.GetConfig(ctx)
+	m.srcListDir = m.makeListDir(ctx, m.Fsrc, m.SrcIncludeAll)
 	if !m.NoTraverse {
-		m.dstListDir = m.makeListDir(m.Fdst, m.DstIncludeAll)
+		m.dstListDir = m.makeListDir(ctx, m.Fdst, m.DstIncludeAll)
 	}
 	// Now create the matching transform
 	// ..normalise the UTF8 first
@@ -65,7 +66,7 @@ func (m *March) init() {
 	//                  | Yes | No  | No                 |
 	//                  | No  | Yes | Yes                |
 	//                  | Yes | Yes | Yes                |
-	if m.Fdst.Features().CaseInsensitive || fs.Config.IgnoreCaseSync {
+	if m.Fdst.Features().CaseInsensitive || ci.IgnoreCaseSync {
 		m.transforms = append(m.transforms, strings.ToLower)
 	}
 }
@@ -75,9 +76,11 @@ type listDirFn func(dir string) (entries fs.DirEntries, err error)
 
 // makeListDir makes constructs a listing function for the given fs
 // and includeAll flags for marching through the file system.
-func (m *March) makeListDir(f fs.Fs, includeAll bool) listDirFn {
-	if !(fs.Config.UseListR && f.Features().ListR != nil) && // !--fast-list active and
-		!(fs.Config.NoTraverse && filter.Active.HaveFilesFrom()) { // !(--files-from and --no-traverse)
+func (m *March) makeListDir(ctx context.Context, f fs.Fs, includeAll bool) listDirFn {
+	ci := fs.GetConfig(ctx)
+	fi := filter.GetConfig(ctx)
+	if !(ci.UseListR && f.Features().ListR != nil) && // !--fast-list active and
+		!(ci.NoTraverse && fi.HaveFilesFrom()) { // !(--files-from and --no-traverse)
 		return func(dir string) (entries fs.DirEntries, err error) {
 			return list.DirSorted(m.Ctx, f, includeAll, dir)
 		}
@@ -95,7 +98,7 @@ func (m *March) makeListDir(f fs.Fs, includeAll bool) listDirFn {
 		mu.Lock()
 		defer mu.Unlock()
 		if !started {
-			dirs, dirsErr = walk.NewDirTree(m.Ctx, f, m.Dir, includeAll, fs.Config.MaxDepth)
+			dirs, dirsErr = walk.NewDirTree(m.Ctx, f, m.Dir, includeAll, ci.MaxDepth)
 			started = true
 		}
 		if dirsErr != nil {
@@ -122,15 +125,17 @@ type listDirJob struct {
 }
 
 // Run starts the matching process off
-func (m *March) Run() error {
-	m.init()
+func (m *March) Run(ctx context.Context) error {
+	ci := fs.GetConfig(ctx)
+	fi := filter.GetConfig(ctx)
+	m.init(ctx)
 
-	srcDepth := fs.Config.MaxDepth
+	srcDepth := ci.MaxDepth
 	if srcDepth < 0 {
 		srcDepth = fs.MaxLevel
 	}
 	dstDepth := srcDepth
-	if filter.Active.Opt.DeleteExcluded {
+	if fi.Opt.DeleteExcluded {
 		dstDepth = fs.MaxLevel
 	}
 
@@ -141,8 +146,9 @@ func (m *March) Run() error {
 	// Start some directory listing go routines
 	var wg sync.WaitGroup         // sync closing of go routines
 	var traversing sync.WaitGroup // running directory traversals
-	in := make(chan listDirJob, fs.Config.Checkers)
-	for i := 0; i < fs.Config.Checkers; i++ {
+	checkers := ci.Checkers
+	in := make(chan listDirJob, checkers)
+	for i := 0; i < checkers; i++ {
 		wg.Add(1)
 		go func() {
 			defer wg.Done()
@@ -376,6 +382,7 @@ func (m *March) processJob(job listDirJob) ([]listDirJob, error) {
 		srcList, dstList       fs.DirEntries
 		srcListErr, dstListErr error
 		wg                     sync.WaitGroup
+		mu                     sync.Mutex
 	)
 
 	// List the src and dst directories
@@ -397,30 +404,49 @@ func (m *March) processJob(job listDirJob) ([]listDirJob, error) {
 	// Wait for listings to complete and report errors
 	wg.Wait()
 	if srcListErr != nil {
-		fs.Errorf(job.srcRemote, "error reading source directory: %v", srcListErr)
+		if job.srcRemote != "" {
+			fs.Errorf(job.srcRemote, "error reading source directory: %v", srcListErr)
+		} else {
+			fs.Errorf(m.Fsrc, "error reading source root directory: %v", srcListErr)
+		}
 		srcListErr = fs.CountError(srcListErr)
 		return nil, srcListErr
 	}
 	if dstListErr == fs.ErrorDirNotFound {
 		// Copy the stuff anyway
 	} else if dstListErr != nil {
-		fs.Errorf(job.dstRemote, "error reading destination directory: %v", dstListErr)
+		if job.dstRemote != "" {
+			fs.Errorf(job.dstRemote, "error reading destination directory: %v", dstListErr)
+		} else {
+			fs.Errorf(m.Fdst, "error reading destination root directory: %v", dstListErr)
+		}
 		dstListErr = fs.CountError(dstListErr)
 		return nil, dstListErr
 	}
 
 	// If NoTraverse is set, then try to find a matching object
-	// for each item in the srcList
+	// for each item in the srcList to head dst object
+	ci := fs.GetConfig(m.Ctx)
+	limiter := make(chan struct{}, ci.Checkers)
 	if m.NoTraverse && !m.NoCheckDest {
 		for _, src := range srcList {
-			if srcObj, ok := src.(fs.Object); ok {
-				leaf := path.Base(srcObj.Remote())
-				dstObj, err := m.Fdst.NewObject(m.Ctx, path.Join(job.dstRemote, leaf))
-				if err == nil {
-					dstList = append(dstList, dstObj)
+			wg.Add(1)
+			limiter <- struct{}{}
+			go func(limiter chan struct{}, src fs.DirEntry) {
+				defer wg.Done()
+				if srcObj, ok := src.(fs.Object); ok {
+					leaf := path.Base(srcObj.Remote())
+					dstObj, err := m.Fdst.NewObject(m.Ctx, path.Join(job.dstRemote, leaf))
+					if err == nil {
+						mu.Lock()
+						dstList = append(dstList, dstObj)
+						mu.Unlock()
+					}
 				}
-			}
+				<-limiter
+			}(limiter, src)
 		}
+		wg.Wait()
 	}
 
 	// Work out what to do and do it
