@@ -25,7 +25,9 @@ func Fuzz(data []byte) int {
 		panic(fmt.Sprintf("error decoding %q, input %q: %v", enc, string(data), err))
 	}
 	if !bytes.Equal(data, []byte(decoded)) {
-		panic(fmt.Sprintf("decode mismatch, encoded: %q, org: %q, got: %q", enc, string(data), decoded))
+		table := decodeMap[enc[0]]
+		table--
+		panic(fmt.Sprintf("decode mismatch, encoded: %q, org: %q, got: %q, table %d", enc, string(data), decoded, int(table)))
 	}
 
 	// Everything is good.
