@@ -13,6 +13,7 @@ import (
 	"sync"
 	"time"
 
+	sysdnotify "github.com/iguanesolutions/go-systemd/v5/notify"
 	"github.com/pkg/errors"
 	"github.com/rclone/rclone/fs"
 	fscache "github.com/rclone/rclone/fs/cache"
@@ -55,7 +56,7 @@ type Cache struct {
 	used          int64            // total size of files in the cache
 	outOfSpace    bool             // out of space
 	cleanerKicked bool             // some thread kicked the cleaner upon out of space
-	kickerMu      sync.Mutex       // mutex for clearnerKicked
+	kickerMu      sync.Mutex       // mutex for cleanerKicked
 	kick          chan struct{}    // channel for kicking clear to start
 
 }
@@ -68,38 +69,43 @@ type Cache struct {
 // go into the directory tree.
 type AddVirtualFn func(remote string, size int64, isDir bool) error
 
-// New creates a new cache heirachy for fremote
+// New creates a new cache hierarchy for fremote
 //
 // This starts background goroutines which can be cancelled with the
 // context passed in.
 func New(ctx context.Context, fremote fs.Fs, opt *vfscommon.Options, avFn AddVirtualFn) (*Cache, error) {
+	fName := fremote.Name()
 	fRoot := filepath.FromSlash(fremote.Root())
 	if runtime.GOOS == "windows" {
 		if strings.HasPrefix(fRoot, `\\?`) {
 			fRoot = fRoot[3:]
 		}
 		fRoot = strings.Replace(fRoot, ":", "", -1)
+		// Replace leading ':' if remote was created on the fly as ":backend:/path" as it is illegal in Windows
+		if fName[0] == ':' {
+			fName = "^" + fName[1:]
+		}
 	}
 	cacheDir := config.CacheDir
 	cacheDir, err := filepath.Abs(cacheDir)
 	if err != nil {
 		return nil, errors.Wrap(err, "failed to make --cache-dir absolute")
 	}
-	root := file.UNCPath(filepath.Join(cacheDir, "vfs", fremote.Name(), fRoot))
+	root := file.UNCPath(filepath.Join(cacheDir, "vfs", fName, fRoot))
 	fs.Debugf(nil, "vfs cache: root is %q", root)
-	metaRoot := file.UNCPath(filepath.Join(cacheDir, "vfsMeta", fremote.Name(), fRoot))
+	metaRoot := file.UNCPath(filepath.Join(cacheDir, "vfsMeta", fName, fRoot))
 	fs.Debugf(nil, "vfs cache: metadata root is %q", root)
 
-	fcache, err := fscache.Get(root)
+	fcache, err := fscache.Get(ctx, root)
 	if err != nil {
 		return nil, errors.Wrap(err, "failed to create cache remote")
 	}
-	fcacheMeta, err := fscache.Get(root)
+	fcacheMeta, err := fscache.Get(ctx, root)
 	if err != nil {
 		return nil, errors.Wrap(err, "failed to create cache meta remote")
 	}
 
-	hashType, hashOption := operations.CommonHash(fcache, fremote)
+	hashType, hashOption := operations.CommonHash(ctx, fcache, fremote)
 
 	c := &Cache{
 		fremote:    fremote,
@@ -129,7 +135,7 @@ func New(ctx context.Context, fremote fs.Fs, opt *vfscommon.Options, avFn AddVir
 	}
 
 	// Remove any empty directories
-	c.purgeEmptyDirs()
+	c.purgeEmptyDirs("", true)
 
 	// Create a channel for cleaner to be kicked upon out of space con
 	c.kick = make(chan struct{}, 1)
@@ -231,7 +237,8 @@ func (c *Cache) InUse(name string) bool {
 	return item.inUse()
 }
 
-// DirtyItem the Item if it exists in the cache and is Dirty
+// DirtyItem returns the Item if it exists in the cache **and** is
+// dirty otherwise it returns nil.
 //
 // name should be a remote path not an osPath
 func (c *Cache) DirtyItem(name string) (item *Item) {
@@ -337,10 +344,53 @@ func (c *Cache) Rename(name string, newName string, newObj fs.Object) (err error
 	return nil
 }
 
+// DirExists checks to see if the directory exists in the cache or not.
+func (c *Cache) DirExists(name string) bool {
+	path := c.toOSPath(name)
+	_, err := os.Stat(path)
+	return err == nil
+}
+
+// DirRename the dir in cache
+func (c *Cache) DirRename(oldDirName string, newDirName string) (err error) {
+	// Make sure names are / suffixed for reading keys out of c.item
+	if !strings.HasSuffix(oldDirName, "/") {
+		oldDirName += "/"
+	}
+	if !strings.HasSuffix(newDirName, "/") {
+		newDirName += "/"
+	}
+
+	// Find all items to rename
+	var renames []string
+	c.mu.Lock()
+	for itemName := range c.item {
+		if strings.HasPrefix(itemName, oldDirName) {
+			renames = append(renames, itemName)
+		}
+	}
+	c.mu.Unlock()
+
+	// Rename the items
+	for _, itemName := range renames {
+		newPath := newDirName + itemName[len(oldDirName):]
+		renameErr := c.Rename(itemName, newPath, nil)
+		if renameErr != nil {
+			err = renameErr
+		}
+	}
+
+	// Old path should be empty now so remove it
+	c.purgeEmptyDirs(oldDirName[:len(oldDirName)-1], false)
+
+	fs.Infof(oldDirName, "vfs cache: renamed dir in cache to %q", newDirName)
+	return err
+}
+
 // Remove should be called if name is deleted
 //
 // This returns true if the file was in the transfer queue so may not
-// have completedly uploaded yet.
+// have completely uploaded yet.
 func (c *Cache) Remove(name string) (wasWriting bool) {
 	name = clean(name)
 	c.mu.Lock()
@@ -465,7 +515,7 @@ func (c *Cache) removeNotInUse(item *Item, maxAge time.Duration, emptyOnly bool)
 
 // Retry failed resets during purgeClean()
 func (c *Cache) retryFailedResets() {
-	// Some items may have failed to reset becasue there was not enough space
+	// Some items may have failed to reset because there was not enough space
 	// for saving the cache item's metadata.  Redo the Reset()'s here now that
 	// we may have some available space.
 	if len(c.errItems) != 0 {
@@ -499,7 +549,7 @@ func (c *Cache) purgeClean(quota int64) {
 
 	// Make a slice of clean cache files
 	for _, item := range c.item {
-		if !item.IsDataDirty() {
+		if !item.IsDirty() {
 			items = append(items, item)
 		}
 	}
@@ -548,15 +598,15 @@ func (c *Cache) purgeOld(maxAge time.Duration) {
 }
 
 // Purge any empty directories
-func (c *Cache) purgeEmptyDirs() {
+func (c *Cache) purgeEmptyDirs(dir string, leaveRoot bool) {
 	ctx := context.Background()
-	err := operations.Rmdirs(ctx, c.fcache, "", true)
+	err := operations.Rmdirs(ctx, c.fcache, dir, leaveRoot)
 	if err != nil {
-		fs.Errorf(c.fcache, "vfs cache: failed to remove empty directories from cache: %v", err)
+		fs.Errorf(c.fcache, "vfs cache: failed to remove empty directories from cache path %q: %v", dir, err)
 	}
-	err = operations.Rmdirs(ctx, c.fcacheMeta, "", true)
+	err = operations.Rmdirs(ctx, c.fcacheMeta, dir, leaveRoot)
 	if err != nil {
-		fs.Errorf(c.fcache, "vfs cache: failed to remove empty directories from metadata cache: %v", err)
+		fs.Errorf(c.fcache, "vfs cache: failed to remove empty directories from metadata cache path %q: %v", dir, err)
 	}
 }
 
@@ -608,7 +658,7 @@ func (c *Cache) purgeOverQuota(quota int64) {
 }
 
 // clean empties the cache of stuff if it can
-func (c *Cache) clean(removeCleanFiles bool) {
+func (c *Cache) clean(kicked bool) {
 	// Cache may be empty so end
 	_, err := os.Stat(c.root)
 	if os.IsNotExist(err) {
@@ -624,18 +674,17 @@ func (c *Cache) clean(removeCleanFiles bool) {
 		// Remove any files that are over age
 		c.purgeOld(c.opt.CacheMaxAge)
 
+		if int64(c.opt.CacheMaxSize) <= 0 {
+			break
+		}
+
 		// Now remove files not in use until cache size is below quota starting from the
 		// oldest first
 		c.purgeOverQuota(int64(c.opt.CacheMaxSize))
 
-		// removeCleanFiles indicates that we got ENOSPC error
-		// We remove cache files that are not dirty if we are still avove the max cache size
-		if removeCleanFiles {
-			c.purgeClean(int64(c.opt.CacheMaxSize))
-			c.retryFailedResets()
-		} else {
-			break
-		}
+		// Remove cache files that are not dirty if we are still above the max cache size
+		c.purgeClean(int64(c.opt.CacheMaxSize))
+		c.retryFailedResets()
 
 		used := c.updateUsed()
 		if used <= int64(c.opt.CacheMaxSize) && len(c.errItems) == 0 {
@@ -644,7 +693,7 @@ func (c *Cache) clean(removeCleanFiles bool) {
 	}
 
 	// Was kicked?
-	if removeCleanFiles {
+	if kicked {
 		c.kickerMu.Lock() // Make sure this is called with cache mutex unlocked
 		// Reenable io threads to kick me
 		c.cleanerKicked = false
@@ -663,7 +712,12 @@ func (c *Cache) clean(removeCleanFiles bool) {
 	c.mu.Unlock()
 	uploadsInProgress, uploadsQueued := c.writeback.Stats()
 
-	fs.Infof(nil, "vfs cache: cleaned: objects %d (was %d) in use %d, to upload %d, uploading %d, total size %v (was %v)", newItems, oldItems, totalInUse, uploadsQueued, uploadsInProgress, newUsed, oldUsed)
+	stats := fmt.Sprintf("objects %d (was %d) in use %d, to upload %d, uploading %d, total size %v (was %v)",
+		newItems, oldItems, totalInUse, uploadsQueued, uploadsInProgress, newUsed, oldUsed)
+	fs.Infof(nil, "vfs cache: cleaned: %s", stats)
+	if err = sysdnotify.Status(fmt.Sprintf("[%s] vfs cache: %s", time.Now().Format("15:04"), stats)); err != nil {
+		fs.Errorf(nil, "vfs cache: updating systemd status with current stats failed: %s", err)
+	}
 }
 
 // cleaner calls clean at regular intervals and upon being kicked for out-of-space condition
@@ -682,9 +736,9 @@ func (c *Cache) cleaner(ctx context.Context) {
 	for {
 		select {
 		case <-c.kick: // a thread encountering ENOSPC kicked me
-			c.clean(true) // remove inUse files that are clean (!item.info.Dirty)
+			c.clean(true) // kicked is true
 		case <-timer.C:
-			c.clean(false) // do not remove inUse files
+			c.clean(false) // timer driven cache poll, kicked is false
 		case <-ctx.Done():
 			fs.Debugf(nil, "vfs cache: cleaner exiting")
 			return
