@@ -26,6 +26,9 @@ const (
 	backgroundKickerInterval = 5 * time.Second
 	// maximum number of errors before declaring dead
 	maxErrorCount = 10
+	// If a downloader is within this range or --buffer-size
+	// whichever is the larger, we will reuse the downloader
+	minWindow = 1024 * 1024
 )
 
 // Item is the interface that an item to download must obey
@@ -283,7 +286,7 @@ func (dls *Downloaders) _ensureDownloader(r ranges.Range) (err error) {
 	// defer log.Trace(dls.src, "r=%v", r)("err=%v", &err)
 
 	// The window includes potentially unread data in the buffer
-	window := int64(fs.Config.BufferSize)
+	window := int64(fs.GetConfig(context.TODO()).BufferSize)
 
 	// Increase the read range by the read ahead if set
 	if dls.opt.ReadAhead > 0 {
@@ -298,7 +301,7 @@ func (dls *Downloaders) _ensureDownloader(r ranges.Range) (err error) {
 	r = dls.item.FindMissing(r)
 
 	// If the range is entirely present then we only need to start a
-	// dowloader if the window isn't full.
+	// downloader if the window isn't full.
 	startNew := true
 	if r.IsEmpty() {
 		// Make a new range which includes the window
@@ -327,6 +330,11 @@ func (dls *Downloaders) _ensureDownloader(r ranges.Range) (err error) {
 		r.Size = 0
 	}
 
+	// If buffer size is less than minWindow then make it that
+	if window < minWindow {
+		window = minWindow
+	}
+
 	var dl *downloader
 	// Look through downloaders to find one in range
 	// If there isn't one then start a new one
@@ -521,7 +529,7 @@ func (dl *downloader) open(offset int64) (err error) {
 	// if offset > 0 {
 	// 	rangeOption = &fs.RangeOption{Start: offset, End: size - 1}
 	// }
-	// in0, err := operations.NewReOpen(dl.dls.ctx, dl.dls.src, fs.Config.LowLevelRetries, dl.dls.item.c.hashOption, rangeOption)
+	// in0, err := operations.NewReOpen(dl.dls.ctx, dl.dls.src, ci.LowLevelRetries, dl.dls.item.c.hashOption, rangeOption)
 
 	in0 := chunkedreader.New(context.TODO(), dl.dls.src, int64(dl.dls.opt.ChunkSize), int64(dl.dls.opt.ChunkSizeLimit))
 	_, err = in0.Seek(offset, 0)
@@ -553,7 +561,7 @@ func (dl *downloader) close(inErr error) (err error) {
 		dl.in = nil
 	}
 	if dl.tr != nil {
-		dl.tr.Done(inErr)
+		dl.tr.Done(dl.dls.ctx, inErr)
 		dl.tr = nil
 	}
 	dl._closed = true
@@ -561,7 +569,7 @@ func (dl *downloader) close(inErr error) (err error) {
 	return nil
 }
 
-// closed returns true if the downloader has been closed alread
+// closed returns true if the downloader has been closed already
 func (dl *downloader) closed() bool {
 	dl.mu.Lock()
 	defer dl.mu.Unlock()
@@ -585,7 +593,7 @@ func (dl *downloader) _stop() {
 
 	// stop the downloader by stopping the async reader buffering
 	// any more input. This causes all the stuff in the async
-	// buffer (which can be many MB) to be written to the disk
+	// buffer (which can be many MiB) to be written to the disk
 	// before exiting.
 	if dl.in != nil {
 		dl.in.StopBuffering()
